## Produces various historical simulations

source("common.R")
source("ma.R")

data(exports)

## Calcule l'indice de variabilité
volat <- function(x)
{
  if (any(is.na(x)))
    warning("NAs detected...")

  sd(diff(log(x)), na.rm = TRUE)
}

## Fonction à tout faire pour les simulations historiques
## "i" est le numéro de ligne dans "dataset" à traiter
## "graph" indique si on veut que différents graphiques soient affichés
## "euv" indique si on veut utiliser les prix à l'exportation plutôt que les prix internationaux
hist.sim <- function(i, dataset = read.csv("hist-sim.csv", as.is = TRUE), n = 5, graph = TRUE, euv = FALSE, r = 0.05)
{
  exports.code <- dataset[i, "exports.code"]
  comm <- dataset[i, "comm"]
  fao.file <- dataset[i, "fao.file"]
  line.volume <- dataset[i, "line.volume"]
  line.value <- dataset[i, "line.value"]
  start.year <- dataset[i, "start.year"]
  end.year <- dataset[i, "end.year"]

  deflator <- window(make.annual.ma(usppi[[1]] / ref.price), start = start.year, end = end.year)

  total.exports <- window(exports[, exports.code], start = start.year, end = end.year) * 10^9

  fao <- read.csv(fao.file)
  volumes <- ts(as.numeric(fao[line.volume, (5+start.year-1961):(5+end.year-1961)]), start = start.year, frequency = 1)
  values <- ts(as.numeric(fao[line.value, (5+start.year-1961):(5+end.year-1961)]), start = start.year, frequency = 1) * 1000

  if (euv)
    {
      prices <- values / volumes
    }
  else
    {
      prices <- window(make.annual.ma(na.contiguous(commodities[[comm]])), start = start.year, end = end.year)
      ## Conversion des ¢/lb en $/tonne
      if (comm == "COTTON.LIVERPOOL" || comm == "COFFEE.OTHER.MILDS")
        {
          prices <- prices * 22.04586
        }
    }

  if (any(is.na(prices)))
    stop("NAs in the prices...")

  correction <- - ma.annual(prices, n, 1, diff = TRUE) * volumes

  prices <- prices / deflator
  correction <- correction / deflator
  values <- values / deflator
  total.exports <- total.exports / deflator

  if (graph)
    {
      x11()
      plot(total.exports, main = "Exports totaux")
      lines(total.exports + correction, col = "red")
      lines(values, col = "blue")

      x11()
      plot(values, main = "Exports de la commodité")
      lines(values + correction, col = "red")

      x11()
      plot(values/volumes, main = "Prix")
      lines(prices, col = "red")
      lines(ma.annual(prices, 5, 1), col = "blue")

      x11()
      plot(volumes, main = "Volumes")
    }

  cost <- max(cumsum(correction * (1+r)^((2003 - start.year):(2003 - end.year))))

  list(volat.total.before = volat(total.exports),
       volat.total.after = volat(total.exports + correction),
       volat.comm.before = volat(values),
       volat.comm.after = volat(values + correction),
       cost = cost,
       exports = values[end.year - start.year + 1],
       exports.ratio = values[end.year - start.year + 1] / total.exports[end.year - start.year + 1],
       export.unit.price.corr = cor.test(values/volumes, prices),
       export.unit.price.diff.mean = mean(log((values/volumes)/prices), na.rm = TRUE),
       export.unit.price.diff.sd = sd(log((values/volumes)/prices), na.rm = TRUE),
       basis.price.corr = cor.test(values/volumes - prices, prices),
       quantity.trend = mean(diff(log(volumes))),
       quantity.variability = sd(diff(log(volumes))))
}

d3 <- function(x)
{
  formatC(x, digits = 3, format = "f")
}

d3sign <- function(x)
{
  if (x > 0)
    paste("+", d3(x), sep = "")
  else
    d3(x)
}

d2 <- function(x)
{
  formatC(x, digits = 2, format = "f")
}

## Affiche une table LaTeX avec les couples pays/commodités (table 3.6 page 26 of Villemot (2004))
intro.table <- function(dataset = read.csv("hist-sim.csv", as.is = TRUE))
{
  for(i in 1:nrow(dataset))
    {
      x <- hist.sim(i, dataset, graph = FALSE)
      cat(dataset[i, "country"], comm.names[[dataset[i, "comm"]]],
          paste(dataset[i, "start.year"], "-", dataset[i, "end.year"], sep = ""),
          paste(round(x$exports / 10^6), "M\\$"),
          paste(formatC(x$exports.ratio * 100, digits = 1, format = "f"), "\\%", sep = ""),
          sep = " & ")
      cat(" \\\\", fill = TRUE)
    }
}

## Affiche une table LaTeX avec les résultats des simulations
## euv = FALSE: table 3.7 page 27 of Villemot (2004)
## euv = TRUE: table 3.9 page 29 of Villemot (2004)
sim.table <- function(euv = FALSE, dataset = read.csv("hist-sim.csv", as.is = TRUE))
{
  for(i in 1:nrow(dataset))
    {
      x <- hist.sim(i, dataset, euv = euv, graph = FALSE)
      cat(dataset[i, "country"], d3(x$volat.comm.before), d3(x$volat.comm.after), d3sign(x$volat.comm.after - x$volat.comm.before),
          d3(x$volat.total.before), d3(x$volat.total.after), d3sign(x$volat.total.after - x$volat.total.before),
          paste(round(x$cost / 10^6), "M\\$"), sep = " & ")
      cat(" \\\\", fill = TRUE)
    }
}

## Affiche une table LaTeX avec des infos sur le différentiel prix internationaux/export
## (table 3.8 page 28 of Villemot (2004))
basis.table <- function(dataset = read.csv("hist-sim.csv", as.is = TRUE))
{
  for(i in 1:nrow(dataset))
    {
      x <- hist.sim(i, dataset, graph = FALSE)
      cat(dataset[i, "country"], d2(x$export.unit.price.corr$estimate), d2(x$export.unit.price.diff.mean),
          d2(x$export.unit.price.diff.sd), d2(x$basis.price.corr$estimate), sep = " & ")
      cat(" \\\\", fill = TRUE)

      cat(" & [", d2(x$export.unit.price.corr$conf.int[1]), ",", d2(x$export.unit.price.corr$conf.int[2]), "] & & & [",
          d2(x$basis.price.corr$conf.int[1]), "," , d2(x$basis.price.corr$conf.int[2]), "] \\\\", sep = "", fill = TRUE)
    }
}

## Affiche une table LaTeX avec des infos sur les quantités (table 3.10 page 30 of Villemot (2004))
quantity.table <- function(dataset = read.csv("hist-sim.csv", as.is = TRUE))
{
  for(i in 1:nrow(dataset))
    {
      x <- hist.sim(i, dataset, graph = FALSE)
      cat(dataset[i, "country"],
          d3(x$quantity.trend), d3(x$quantity.variability),
          sep = " & ")
      cat(" \\\\", fill = TRUE)
    }
}
